from ..handlers.base import freeAccessProjectIds
from ..gateways.auth import AuthGateway
from ..utils.apiMessage import SharedApiMessageKey, SharedApiCode
from ellipticcurve.ecdsa import Ecdsa
from ellipticcurve.publicKey import PublicKey
from ellipticcurve.signature import Signature


def validateDigitalSignature(func):
    def wrapper(self, **data):
        isInternalAccess = self.getInboundProjectId() not in freeAccessProjectIds
        if isInternalAccess:
            return func(self, **data)

        message = self.getBody()
        language = self.getLanguage()
        signatureKey = "Digital-Signature"
        signatureBase64 = self.getHeaders().get(signatureKey)

        signature= _getSignature(signatureBase64)
        if not signature:
            return self.sendJson(
                self.responseJson().error(
                    code=SharedApiCode.invalidDigitalSignature,
                    messageKey=SharedApiMessageKey.invalidParameter,
                    parameter=signatureKey
                )
            )

        if not _isValidDigitalSignature(signature, message, language, memberId=self.userId()) and \
           not _isValidDigitalSignature(signature, message, language, workspaceId=self.workspaceId()):
            return self.sendJson(
                self.responseJson().error(
                    code=SharedApiCode.invalidDigitalSignature,
                    messageKey=SharedApiMessageKey.invalidDigitalSignature
                )
            )

        return func(self, **data)
    return wrapper


def _isValidDigitalSignature(signature, message, language, memberId=None, workspaceId=None):
    publicKeyPem, error = AuthGateway.getPublicKey(memberId=memberId, workspaceId=workspaceId, language=language)
    if error or not publicKeyPem:
        return False

    return Ecdsa.verify(message=message, signature=signature, publicKey=PublicKey.fromPem(publicKeyPem))


def _getSignature(signatureBase64):
    try:
        return Signature.fromBase64(signatureBase64)
    except:
        return None