from ..utils.metadata import Metadata
from ..handlers.base import freeAccessProjectIds
from ..gateways.workspace import WorkspaceGateway


def signedIn(func):
    def wrapper(self, **data):
        isInternalAccess = self.getInboundProjectId() not in freeAccessProjectIds
        if isInternalAccess:
            return func(self, **data)

        headers = self.getHeaders()
        accessToken = headers.get("Access-Token") or self.getParameter("accessToken") or ""
        user, error = WorkspaceGateway.getUser(accessToken=accessToken, headers=headers, language=self.getLanguage())
        if error:
            return self.sendJson(error)

        self.request.headers.update({
            "User-Id": user["id"],
            "User-Name": user["name"],
            "User-Email": user["email"],
            "User-Roles": ",".join(user["permissions"]),
            "Workspace-Id": user["workspaceId"],
            "Path": "",
            "Microservice-Id": Metadata.getProjectId(),
            "Microservice-Signature": ""
        })

        return func(self, **data)
    return wrapper
