from re import sub, match


class Cnpj:

    regex = r"^\d{2}.\d{3}.\d{3}\/\d{4}-\d{2}$"

    def __init__(self, numbers):
        self.numbers = numbers

    def unformatted(self):
        return sub(r"\D", "", self.numbers)

    def formatted(self):
        numbers = self.numbers
        isNumberPattern = match(r"^[0-9]{14}$", numbers)
        cnpjPattern = "{0}.{1}.{2}/{3}-{4}"
        return cnpjPattern.format(numbers[:2], numbers[2:5], numbers[5:8], numbers[8:12], numbers[12:]) if isNumberPattern else numbers

    def matchPattern(self, pattern=None):
        return match(pattern or Cnpj.regex, self.numbers)

    def isValid(self, pattern=None):

        if not self.matchPattern(pattern or Cnpj.regex): return False

        cnpj = self.unformatted()
        if len(cnpj) != 14: return False

        numbers = map(int, cnpj)
        new = numbers[:12]
        prod = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
        while len(new) < 14:
            r = sum([x * y for (x, y) in zip(new, prod)]) % 11
            if r > 1:
                f = 11 - r
            else:
                f = 0
            new.append(f)
            prod.insert(0, 6)

        return bool(new == numbers)